@@ -32,6 +32,7 @@ from weboob.capabilities.base import empty, BaseObject
 from weboob.capabilities.bugtracker import CapBugTracker, Query, Update, Project, Issue, IssueError
 from weboob.tools.application.repl import ReplApplication, defaultcount
 from weboob.tools.application.formatters.iformatter import IFormatter, PrettyFormatter
+from weboob.tools.compat import basestring, unicode
 from weboob.tools.html import html2text
 from weboob.tools.date import parse_french_date
 
@@ -39,6 +40,12 @@ from weboob.tools.date import parse_french_date
 __all__ = ['BoobTracker']
 
 
+try:
+    input = raw_input
+except NameError:
+    pass
+
+
 class IssueFormatter(IFormatter):
     MANDATORY_FIELDS = ('id', 'project', 'title', 'body', 'author')
 
@@ -71,7 +78,7 @@ class IssueFormatter(IFormatter):
         result += self.format_attr(obj, 'category')
         result += self.format_attr(obj, 'assignee')
         if hasattr(obj, 'fields') and not empty(obj.fields):
-            for key, value in obj.fields.iteritems():
+            for key, value in obj.fields.items():
                 result += self.format_key(key.capitalize(), value)
         if hasattr(obj, 'attachments') and obj.attachments:
             result += '\n%s\n' % self.colored('Attachments:', 'green')
@@ -105,8 +112,8 @@ class IssuesListFormatter(PrettyFormatter):
 
 class BoobTracker(ReplApplication):
     APPNAME = 'boobtracker'
-    VERSION = '1.0'
-    COPYRIGHT = 'Copyright(C) 2011 Romain Bignon'
+    VERSION = '1.4'
+    COPYRIGHT = 'Copyright(C) 2011-YEAR Romain Bignon'
     DESCRIPTION = "Console application allowing to create, edit, view bug tracking issues."
     SHORT_DESCRIPTION = "manage bug tracking issues"
     CAPS = CapBugTracker
@@ -309,7 +316,7 @@ class BoobTracker(ReplApplication):
                                         for o in objects_list])
                 output += 'X-Available-%s: %s\n' % (self.sanitize_key(key), availables)
 
-        for key, value in issue.fields.iteritems():
+        for key, value in issue.fields.items():
             output += '%s: %s\n' % (self.sanitize_key(key), value or '')
             # TODO: Add X-Available-* for lists
 
@@ -394,7 +401,7 @@ class BoobTracker(ReplApplication):
             except ValueError as e:
                 if not self.stdin.isatty():
                     raise
-                raw_input("%s -- Press Enter to continue..." % unicode(e).encode("utf-8"))
+                input("%s -- Press Enter to continue..." % unicode(e).encode("utf-8"))
                 continue
 
             try:
@@ -409,7 +416,7 @@ class BoobTracker(ReplApplication):
             except IssueError as e:
                 if not self.stdin.isatty():
                     raise
-                raw_input("%s -- Press Enter to continue..." % unicode(e).encode("utf-8"))
+                input("%s -- Press Enter to continue..." % unicode(e).encode("utf-8"))
 
     def send_notification(self, email_to, issue):
         text = """Hi,
@@ -465,7 +472,7 @@ Weboob Team
         if len(args) == 2:
             return self._complete_object()
         if len(args) == 3:
-            return dict(self.ISSUE_FIELDS).keys()
+            return list(dict(self.ISSUE_FIELDS).keys())
 
     def do_edit(self, line):
         """
