@@ -24,6 +24,7 @@ from functools import wraps
 import re
 import pickle
 import base64
+from hashlib import sha256
 import zlib
 from functools import reduce
 try:
@@ -36,6 +37,7 @@ from copy import deepcopy
 import inspect
 from datetime import datetime, timedelta
 from dateutil import parser
+from threading import Lock
 
 try:
     import requests
@@ -44,10 +46,10 @@ try:
 except ImportError:
     raise ImportError('Please install python-requests >= 2.0')
 
-from weboob.exceptions import BrowserHTTPSDowngrade, ModuleInstallError
+from weboob.exceptions import BrowserHTTPSDowngrade, ModuleInstallError, BrowserRedirect, BrowserIncorrectPassword
 
 from weboob.tools.log import getLogger
-from weboob.tools.compat import basestring, unicode, urlparse, urljoin
+from weboob.tools.compat import basestring, unicode, urlparse, urljoin, urlencode, parse_qsl
 from weboob.tools.json import json
 
 from .cookies import WeboobCookieJar
@@ -99,6 +101,12 @@ class Browser(object):
     Controls the behavior of get_referrer.
     """
 
+    COOKIE_POLICY = None
+    """
+    Default CookieJar policy.
+    Example: weboob.browser.cookies.BlockAllCookies()
+    """
+
     @classmethod
     def asset(cls, localfile):
         """
@@ -112,6 +120,7 @@ class Browser(object):
         self.logger = getLogger('browser', logger)
         self.responses_dirname = responses_dirname
         self.responses_count = 1
+        self.responses_count_lock = Lock()
 
         if isinstance(self.VERIFY, basestring):
             self.VERIFY = self.asset(self.VERIFY)
@@ -145,11 +154,15 @@ class Browser(object):
             # try to get an extension (and avoid adding 'None')
             ext = mimetypes.guess_extension(mimetype, False) or ''
 
+        with self.responses_count_lock:
+            counter = self.responses_count
+            self.responses_count += 1
+
         path = re.sub(r'[^A-z0-9\.-_]+', '_', urlparse(response.url).path.rpartition('/')[2])[-10:]
         if path.endswith(ext):
             ext = ''
         filename = '%02d-%d%s%s%s' % \
-            (self.responses_count, response.status_code, '-' if path else '', path, ext)
+            (counter, response.status_code, '-' if path else '', path, ext)
 
         response_filepath = os.path.join(self.responses_dirname, filename)
 
@@ -174,7 +187,6 @@ class Browser(object):
         with open(match_filepath, 'a') as f:
             f.write('# %d %s %s\n' % (response.status_code, response.reason, response.headers.get('Content-Type', '')))
             f.write('%s\t%s\n' % (response.url, filename))
-        self.responses_count += 1
 
         msg = u'Response saved to %s' % response_filepath
         if warning:
@@ -225,6 +237,8 @@ class Browser(object):
         self.session = session
 
         session.cookies = WeboobCookieJar()
+        if self.COOKIE_POLICY:
+            session.cookies.set_policy(self.COOKIE_POLICY)
 
     def set_profile(self, profile):
         profile.setup_session(self.session)
@@ -317,6 +331,8 @@ class Browser(object):
             # previous version it doesn't calls extract_cookies_to_jar(), it is
             # not a problem as we keep our own cookiejar instance.
             preq._cookies = WeboobCookieJar.from_cookiejar(preq._cookies)
+            if self.COOKIE_POLICY:
+                preq._cookies.set_policy(self.COOKIE_POLICY)
 
         if proxies is None:
             proxies = self.PROXIES
@@ -378,7 +394,6 @@ class Browser(object):
         # in case we did not catch something that should be
         response.raise_for_status()
 
-
     def build_request(self, url, referrer=None, data_encoding=None, **kwargs):
         """
         Does the same job as open(), but returns a Request without
@@ -393,7 +408,7 @@ class Browser(object):
 
         # guess method
         if req.method is None:
-            if req.data:
+            if req.data or req.json:
                 req.method = 'POST'
             else:
                 req.method = 'GET'
@@ -421,7 +436,7 @@ class Browser(object):
         """
         return self.session.prepare_request(req)
 
-    REFRESH_RE = re.compile(r"^(?P<sleep>[\d\.]+)(; url=[\"']?(?P<url>.*?)[\"']?)?$", re.IGNORECASE)
+    REFRESH_RE = re.compile(r"^(?P<sleep>[\d\.]+)(;\s*url=[\"']?(?P<url>.*?)[\"']?)?$", re.IGNORECASE)
 
     def handle_refresh(self, response):
         """
@@ -661,6 +676,7 @@ class PagesBrowser(DomainBrowser):
         """
 
         callback = kwargs.pop('callback', lambda response: response)
+        page_class = kwargs.pop('page', None)
 
         # Have to define a callback to seamlessly process synchronous and
         # asynchronous requests, see :meth:`Browser.open` and its `is_async`
@@ -668,11 +684,14 @@ class PagesBrowser(DomainBrowser):
         def internal_callback(response):
             # Try to handle the response page with an URL instance.
             response.page = None
+            if page_class:
+                response.page = page_class(self, response)
+                return callback(response)
+
             for url in self._urls.values():
-                page = url.handle(response)
-                if page is not None:
-                    self.logger.debug('Handle %s with %s' % (response.url, page.__class__.__name__))
-                    response.page = page
+                response.page = url.handle(response)
+                if response.page is not None:
+                    self.logger.debug('Handle %s with %s', response.url, response.page.__class__.__name__)
                     break
 
             if response.page is None:
@@ -686,7 +705,7 @@ class PagesBrowser(DomainBrowser):
                     if proto_base == 'https' and proto_response != 'https':
                         raise BrowserHTTPSDowngrade()
 
-                self.logger.debug('Unable to handle %s' % response.url)
+                self.logger.debug('Unable to handle %s', response.url)
 
             return callback(response)
 
@@ -930,3 +949,137 @@ class AbstractBrowser(Browser):
 
         cls.__bases__ = (parent,)
         return object.__new__(cls)
+
+
+class OAuth2Mixin(StatesMixin):
+    AUTHORIZATION_URI = None
+    ACCESS_TOKEN_URI = None
+    SCOPE = ''
+
+    client_id = None
+    client_secret = None
+    redirect_uri = None
+    access_token = None
+    access_token_expire = None
+    auth_uri = None
+    token_type = None
+    refresh_token = None
+
+    def __init__(self, *args, **kwargs):
+        super(OAuth2Mixin, self).__init__(*args, **kwargs)
+        self.__states__ += ('access_token', 'access_token_expire', 'refresh_token', 'token_type')
+
+    def build_request(self, *args, **kwargs):
+        headers = kwargs.setdefault('headers', {})
+        if self.access_token:
+            headers['Authorization'] = '{} {}'.format(self.token_type, self.access_token)
+        return super(OAuth2Mixin, self).build_request(*args, **kwargs)
+
+    def dump_state(self):
+        self.access_token_expire = unicode(self.access_token_expire) if self.access_token_expire else None
+        return super(OAuth2Mixin, self).dump_state()
+
+    def load_state(self, state):
+        super(OAuth2Mixin, self).load_state(state)
+        self.access_token_expire = parser.parse(self.access_token_expire) if self.access_token_expire else None
+
+    @property
+    def logged(self):
+        return self.access_token is not None and self.access_token_expire > datetime.now()
+
+    def do_login(self):
+        if self.refresh_token:
+            self.use_refresh_token()
+        elif self.auth_uri:
+            self.request_access_token(self.auth_uri)
+        else:
+            self.request_authorization()
+
+    def build_authorization_parameters(self):
+        return {'redirect_uri':    self.redirect_uri,
+                'scope':           self.SCOPE,
+                'client_id':       self.client_id,
+                'response_type':   'code',
+               }
+
+    def build_authorization_uri(self):
+        p = urlparse(self.AUTHORIZATION_URI)
+        q = dict(parse_qsl(p.query))
+        q.update(self.build_authorization_parameters())
+        return p._replace(query=urlencode(q)).geturl()
+
+    def request_authorization(self):
+        raise BrowserRedirect(self.build_authorization_uri())
+
+    def build_access_token_parameters(self, values):
+        return {'code':             values['code'],
+                'grant_type':       'authorization_code',
+                'redirect_uri':     self.redirect_uri,
+                'client_id':        self.client_id,
+                'client_secret':    self.client_secret,
+                }
+
+    def do_token_request(self, data):
+        return self.open(self.ACCESS_TOKEN_URI, data=data)
+
+    def request_access_token(self, auth_uri):
+        if isinstance(auth_uri, dict):
+            values = auth_uri
+        else:
+            values = dict(parse_qsl(urlparse(auth_uri).query))
+        data = self.build_access_token_parameters(values)
+        try:
+            auth_response = self.do_token_request(data).json()
+        except ClientError:
+            raise BrowserIncorrectPassword()
+
+        self.update_token(auth_response)
+
+    def use_refresh_token(self):
+        data = {'grant_type':       'refresh_token',
+                'refresh_token':    self.refresh_token,
+               }
+        try:
+            auth_response = self.do_token_request(data).json()
+        except ClientError:
+            raise BrowserIncorrectPassword()
+
+        self.update_token(auth_response)
+
+    def update_token(self, auth_response):
+        self.token_type = auth_response['token_type'].capitalize() # don't know yet if this is a good idea, but required by bnpstet
+        if 'refresh_token' in auth_response:
+            self.refresh_token = auth_response['refresh_token']
+        self.access_token = auth_response['access_token']
+        self.access_token_expire = datetime.now() + timedelta(seconds=int(auth_response['expires_in']))
+
+
+class OAuth2PKCEMixin(OAuth2Mixin):
+    def __init__(self, *args, **kwargs):
+        super(OAuth2PKCEMixin, self).__init__(*args, **kwargs)
+        self.__states__ += ('pkce_verifier', 'pkce_challenge')
+        self.pkce_verifier = self.code_verifier()
+        self.pkce_challenge = self.code_challenge(self.pkce_verifier)
+
+    # PKCE (Proof Key for Code Exchange) standard protocol methods:
+    def code_verifier(self, bytes_number=64):
+        return base64.urlsafe_b64encode(os.urandom(bytes_number)).rstrip(b'=')
+
+    def code_challenge(self, verifier):
+        digest = sha256(verifier).hexdigest()
+        return base64.urlsafe_b64encode(digest)
+
+    def build_authorization_parameters(self):
+        return {'redirect_uri':    self.redirect_uri,
+                'code_challenge':  self.pkce_challenge,
+                'client_id':       self.client_id
+               }
+
+    def build_access_token_parameters(self, values):
+        return {'code':             values['code'],
+                'grant_type':       'authorization_code',
+                'code_verifier':    self.pkce_verifier,
+                'redirect_uri':     self.redirect_uri,
+                'client_id':        self.client_id,
+                'client_secret':    self.client_secret,
+                }
