@@ -26,21 +26,30 @@ import re
 import sys
 import os
 import subprocess
+import hashlib
 from datetime import datetime
 from contextlib import closing
 from compileall import compile_dir
-from io import BytesIO
+from io import BytesIO, StringIO
 
-from weboob.exceptions import BrowserHTTPError, BrowserHTTPNotFound
+from weboob.exceptions import BrowserHTTPError, BrowserHTTPNotFound, ModuleInstallError
 from .modules import LoadedModule
 from weboob.tools.log import getLogger
-from weboob.tools.misc import to_unicode
+from weboob.tools.misc import get_backtrace, to_unicode, find_exe
+from weboob.tools.compat import basestring, unicode
 try:
     from ConfigParser import RawConfigParser, DEFAULTSECT
 except ImportError:
     from configparser import RawConfigParser, DEFAULTSECT
 
 
+def open_for_config(filename):
+    if sys.version_info.major == 2:
+        return open(filename, 'wb')
+    else:
+        return open(filename, 'w', encoding='utf-8')
+
+
 class ModuleInfo(object):
     """
     Information about a module available on a repository.
@@ -71,9 +80,10 @@ class ModuleInfo(object):
         self.icon = items['icon'].strip() or None
         self.urls = items['urls']
 
-    def has_caps(self, caps):
-        if not isinstance(caps, (list, tuple)):
-            caps = [caps]
+    def has_caps(self, *caps):
+        """Return True if module implements at least one of the caps."""
+        if len(caps) == 1 and isinstance(caps[0], (list, tuple)):
+            caps = caps[0]
         for c in caps:
             if type(c) == type:
                 c = c.__name__
@@ -120,6 +130,9 @@ class Repository(object):
         self.local = None
         self.signed = False
         self.key_update = 0
+        self.obsolete = False
+        self.logger = getLogger('repository')
+        self.errors = {}
 
         self.modules = {}
 
@@ -154,33 +167,36 @@ class Repository(object):
         :param repo_path: path to save the downloaded index file.
         :type repo_path: str
         """
+        built = False
         if self.local:
             # Repository is local, open the file.
             filename = os.path.join(self.localurl2path(), self.INDEX)
             try:
                 fp = open(filename, 'r')
-            except IOError as e:
+            except IOError:
                 # This local repository doesn't contain a built modules.list index.
                 self.name = Repositories.url2filename(self.url)
                 self.build_index(self.localurl2path(), filename)
+                built = True
                 fp = open(filename, 'r')
         else:
             # This is a remote repository, download file
             try:
-                fp = BytesIO(browser.open(posixpath.join(self.url, self.INDEX)).content)
+                fp = StringIO(browser.open(posixpath.join(self.url, self.INDEX)).text)
             except BrowserHTTPError as e:
                 raise RepositoryUnavailable(unicode(e))
 
         self.parse_index(fp)
 
-        if self.local:
+        # this value can be changed by parse_index
+        if self.local and not built:
             # Always rebuild index of a local repository.
             self.build_index(self.localurl2path(), filename)
 
         # Save the repository index in ~/.weboob/repositories/
         self.save(repo_path, private=True)
 
-    def retrieve_keyring(self, browser, keyring_path):
+    def retrieve_keyring(self, browser, keyring_path, progress):
         # ignore local
         if self.local:
             return
@@ -202,11 +218,11 @@ class Repository(object):
             if keyring.exists():
                 if not keyring.is_valid(keyring_data, sig_data):
                     raise InvalidSignature('the keyring itself')
-                print('The keyring was updated (and validated by the previous one).')
-            else:
-                print('First time saving the keyring, blindly accepted.')
+                progress.progress(0.0, 'The keyring was updated (and validated by the previous one).')
+            elif not progress.prompt('The repository %s isn\'t trusted yet.\nFingerprint of keyring is %s\nAre you sure you want to continue?' % (self.url, hashlib.sha1(keyring_data).hexdigest())):
+                raise RepositoryUnavailable('Repository not trusted')
             keyring.save(keyring_data, self.key_update)
-            print(keyring)
+            progress.progress(0.0, str(keyring))
 
     def parse_index(self, fp):
         """
@@ -226,6 +242,7 @@ class Repository(object):
             self.maintainer = items['maintainer']
             self.signed = bool(int(items.get('signed', '0')))
             self.key_update = int(items.get('key_update', '0'))
+            self.obsolete = bool(int(items.get('obsolete', '0')))
         except KeyError as e:
             raise RepositoryUnavailable('Missing global parameters in repository: %s' % e)
         except ValueError as e:
@@ -260,8 +277,9 @@ class Repository(object):
         :param filename: file to save index
         :type filename: str
         """
-        print('Rebuild index')
+        self.logger.debug('Rebuild index')
         self.modules.clear()
+        self.errors.clear()
 
         if os.path.isdir(os.path.join(path, self.KEYDIR)):
             self.signed = True
@@ -272,7 +290,7 @@ class Repository(object):
 
         for name in sorted(os.listdir(path)):
             module_path = os.path.join(path, name)
-            if not os.path.isdir(module_path) or '.' in name or name == self.KEYDIR:
+            if not os.path.isdir(module_path) or '.' in name or name == self.KEYDIR or not os.path.exists(os.path.join(module_path, '__init__.py')):
                 continue
 
             try:
@@ -283,7 +301,10 @@ class Repository(object):
                     if fp:
                         fp.close()
             except Exception as e:
-                print('Unable to build module %s: [%s] %s' % (name, type(e).__name__, e), file=sys.stderr)
+                self.logger.warning('Unable to build module %s: [%s] %s' % (name, type(e).__name__, e))
+                bt = get_backtrace(e)
+                self.logger.debug(bt)
+                self.errors[name] = bt
             else:
                 m = ModuleInfo(module.name)
                 m.version = self.get_tree_mtime(module_path)
@@ -329,12 +350,16 @@ class Repository(object):
         if private:
             config.set(DEFAULTSECT, 'url', self.url)
 
-        for module in self.modules.itervalues():
+        for module in self.modules.values():
             config.add_section(module.name)
             for key, value in module.dump():
-                config.set(module.name, key, to_unicode(value).encode('utf-8'))
+                if sys.version_info.major == 2:
+                    # python2's configparser enforces bytes coercion with str(value)...
+                    config.set(module.name, key, to_unicode(value).encode('utf-8'))
+                else:
+                    config.set(module.name, key, value)
 
-        with open(filename, 'wb') as f:
+        with open_for_config(filename) as f:
             config.write(f)
 
 
@@ -365,9 +390,10 @@ class Versions(object):
 
     def save(self):
         config = RawConfigParser()
-        for name, version in self.versions.iteritems():
+        for name, version in self.versions.items():
             config.set(DEFAULTSECT, name, version)
-        with open(os.path.join(self.path, self.VERSIONS_LIST), 'wb') as fp:
+
+        with open_for_config(os.path.join(self.path, self.VERSIONS_LIST)) as fp:
             config.write(fp)
 
 
@@ -378,20 +404,23 @@ class IProgress(object):
     def error(self, message):
         raise NotImplementedError()
 
+    def prompt(self, message):
+        raise NotImplementedError()
+
     def __repr__(self):
         return '<%s>' % self.__class__.__name__
 
 
 class PrintProgress(IProgress):
     def progress(self, percent, message):
-        print('=== [%3.0f%%] %s' % (percent*100, message))
+        print('=== [%3.0f%%] %s' % (percent*100, message), file=sys.stderr)
 
     def error(self, message):
         print('ERROR: %s' % message, file=sys.stderr)
 
-
-class ModuleInstallError(Exception):
-    pass
+    def prompt(self, message):
+        print('%s (Y/n): *** ASSUMING YES ***' % message, file=sys.stderr)
+        return True
 
 
 DEFAULT_SOURCES_LIST = \
@@ -400,7 +429,7 @@ DEFAULT_SOURCES_LIST = \
 # The entries below override the entries above (with
 # backends of the same name).
 
-http://updates.weboob.org/%(version)s/main/
+https://updates.weboob.org/%(version)s/main/
 
 # DEVELOPMENT
 # If you want to hack on Weboob modules, you may add a
@@ -443,7 +472,7 @@ class Repositories(object):
         self.repositories = []
 
         if not os.path.exists(self.sources_list):
-            with open(self.sources_list, 'w') as f:
+            with open_for_config(self.sources_list) as f:
                 f.write(DEFAULT_SOURCES_LIST)
             self.update()
         else:
@@ -452,11 +481,14 @@ class Repositories(object):
     def load_browser(self):
         from weboob.browser.browsers import Browser
         from weboob.browser.profiles import Weboob as WeboobProfile
+        from weboob.tools.compat import getproxies
 
         class WeboobBrowser(Browser):
             PROFILE = WeboobProfile(self.version)
         if self.browser is None:
-            self.browser = WeboobBrowser()
+            self.browser = WeboobBrowser(
+                logger=getLogger('browser', parent=self.logger),
+                proxy=getproxies())
 
     def create_dir(self, name):
         if not os.path.exists(name):
@@ -482,7 +514,7 @@ class Repositories(object):
         """
         modules = {}
         for repos in reversed(self.repositories):
-            for name, info in repos.modules.iteritems():
+            for name, info in repos.modules.items():
                 if name not in modules and (not caps or info.has_caps(caps)):
                     modules[name] = self._extend_module_info(repos, info)
         return modules
@@ -568,7 +600,7 @@ class Repositories(object):
         for name in os.listdir(self.repos_dir):
             os.remove(os.path.join(self.repos_dir, name))
 
-        gpgv = Keyring.find_gpgv()
+        gpg_found = Keyring.find_gpg() or Keyring.find_gpgv()
         for line in self._parse_source_list():
             progress.progress(0.0, 'Getting %s' % line)
             repository = Repository(line)
@@ -578,15 +610,19 @@ class Repositories(object):
             keyring_path = os.path.join(self.keyrings_dir, filename)
             try:
                 repository.retrieve_index(self.browser, repo_path)
-                if gpgv:
-                    repository.retrieve_keyring(self.browser, keyring_path)
+                if gpg_found:
+                    repository.retrieve_keyring(self.browser, keyring_path, progress)
                 else:
-                    progress.error('Cannot find gpgv to check for repository authenticity.\n'
-                                    'You should install GPG for better security.')
+                    progress.error('Cannot find gpg or gpgv to check for repository authenticity.\n'
+                                   'You should install GPG for better security.')
             except RepositoryUnavailable as e:
                 progress.error('Unable to load repository: %s' % e)
             else:
                 self.repositories.append(repository)
+                if repository.obsolete:
+                    last_update = datetime.strptime(str(repository.update), '%Y%m%d%H%M').strftime('%Y-%m-%d')
+                    progress.error('This repository does not receive updates anymore (since %s).\n'
+                                   'Your weboob version is probably obsolete and should be upgraded.' % last_update)
 
     def check_repositories(self):
         """
@@ -610,12 +646,17 @@ class Repositories(object):
         :param progress: observer object.
         :type progress: :class:`IProgress`
         """
-        self.update_repositories()
+        self.update_repositories(progress)
 
         to_update = []
-        for name, info in self.get_all_modules_info().iteritems():
+        for name, info in self.get_all_modules_info().items():
             if not info.is_local() and info.is_installed():
-                to_update.append(info)
+                if self.versions.get(name) != info.version:
+                    to_update.append(info)
+
+        if len(to_update) == 0:
+            progress.progress(1.0, 'All modules are up-to-date.')
+            return
 
         class InstallProgress(PrintProgress):
             def __init__(self, n):
@@ -674,7 +715,7 @@ class Repositories(object):
             raise ModuleInstallError('Unable to fetch module: %s' % e)
 
         # Check signature
-        if module.signed and Keyring.find_gpgv():
+        if module.signed and (Keyring.find_gpg() or Keyring.find_gpgv()):
             progress.progress(0.5, 'Checking module authenticity...')
             sig_data = self.browser.open(posixpath.join(module.url + '.sig')).content
             keyring_path = os.path.join(self.keyrings_dir, self.url2filename(module.repo_url))
@@ -711,11 +752,22 @@ class Repositories(object):
         """
         return ''.join([l if l.isalnum() else '_' for l in url])
 
+    def __iter__(self):
+        for repository in self.repositories:
+            yield repository
+
+    @property
+    def errors(self):
+        errors = {}
+        for repository in self:
+            errors.update(repository.errors)
+        return errors
+
 
 class InvalidSignature(Exception):
     def __init__(self, filename):
         self.filename = filename
-        Exception.__init__(self, 'Invalid signature for %s' % filename)
+        super(InvalidSignature, self).__init__('Invalid signature for %s' % filename)
 
 
 class Keyring(object):
@@ -741,7 +793,7 @@ class Keyring(object):
         if os.path.exists(self.path):
             # Check the file is not empty.
             # This is because there was a bug creating empty keyring files.
-            with open(self.path, 'r') as fp:
+            with open(self.path, 'rb') as fp:
                 if len(fp.read().strip()):
                     return True
         return False
@@ -750,50 +802,69 @@ class Keyring(object):
         with open(self.path, 'wb') as fp:
             fp.write(keyring_data)
         self.version = version
-        with open(self.vpath, 'wb') as fp:
+        with open_for_config(self.vpath) as fp:
             fp.write(str(version))
 
     @staticmethod
     def find_gpgv():
-        if os.getenv('GPGV_EXECUTABLE'):
-            return os.getenv('GPGV_EXECUTABLE')
-        paths = os.getenv('PATH', os.defpath).split(os.pathsep)
-        for path in paths:
-            for ex in ('gpgv2', 'gpgv'):
-                fpath = os.path.join(path, ex)
-                if os.path.exists(fpath) and os.access(fpath, os.X_OK):
-                    return fpath
+        return find_exe('gpgv2') or find_exe('gpgv')
+
+    @staticmethod
+    def find_gpg():
+        return find_exe('gpg2') or find_exe('gpg')
 
     def is_valid(self, data, sigdata):
         """
         Check if the data is signed by an accepted key.
         data and sigdata should be strings.
         """
+        gpg = self.find_gpg()
         gpgv = self.find_gpgv()
+
+        if gpg:
+            from tempfile import mkdtemp
+            gpg_homedir = mkdtemp(prefix='weboob_gpg_')
+            verify_command = [gpg, '--verify', '--no-options',
+                              '--no-default-keyring', '--quiet',
+                              '--homedir', gpg_homedir]
+        elif gpgv:
+            verify_command = [gpgv]
+
         from tempfile import NamedTemporaryFile
-        with NamedTemporaryFile(suffix='.sig') as sigfile:
-            sigfile.write(sigdata)
-            sigfile.flush()  # very important
-            assert isinstance(data, basestring)
-            # Yes, all of it is necessary
-            proc = subprocess.Popen([gpgv,
-                    '--status-fd', '1',
-                    '--keyring', os.path.realpath(self.path),
-                    os.path.realpath(sigfile.name),
-                    '-'],
-                stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE)
-            out, err = proc.communicate(data)
-            if proc.returncode or 'GOODSIG' not in out or 'VALIDSIG' not in out:
+        with NamedTemporaryFile(suffix='.sig', delete=False) as sigfile:
+            temp_filename = sigfile.name
+            return_code = None
+            out = ''
+            err = ''
+            try:
+                sigfile.write(sigdata)
+                sigfile.flush()  # very important
+                sigfile.close()
+                assert isinstance(data, bytes)
+                # Yes, all of it is necessary
+                proc = subprocess.Popen(verify_command + [
+                        '--status-fd', '1',
+                        '--keyring', os.path.realpath(self.path),
+                        os.path.realpath(sigfile.name),
+                        '-'],
+                    stdin=subprocess.PIPE,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE)
+                out, err = proc.communicate(data)
+                return_code = proc.returncode
+            finally:
+                os.unlink(temp_filename)
+                if gpg:
+                    shutil.rmtree(gpg_homedir)
+
+            if return_code or b'GOODSIG' not in out or b'VALIDSIG' not in out:
                 print(out, err, file=sys.stderr)
                 return False
         return True
 
     def __str__(self):
         if self.exists():
-            with open(self.vpath, 'r') as f:
-                import hashlib
+            with open(self.path, 'rb') as f:
                 h = hashlib.sha1(f.read()).hexdigest()
             return 'Keyring version %s, checksum %s' % (self.version, h)
         return 'NO KEYRING'
