@@ -20,22 +20,26 @@
 
 from __future__ import print_function
 
+from collections import OrderedDict
 from copy import copy
 import getpass
 import logging
 import subprocess
+from subprocess import check_output
 import sys
 import os
 
 from weboob.capabilities import UserError
 from weboob.capabilities.account import CapAccount, Account, AccountRegisterError
 from weboob.core.backendscfg import BackendAlreadyExists
-from weboob.core.modules import ModuleLoadError
-from weboob.core.repositories import ModuleInstallError
-from weboob.exceptions import BrowserUnavailable, BrowserIncorrectPassword, BrowserForbidden, BrowserSSLError
+from weboob.core.repositories import IProgress
+from weboob.exceptions import BrowserUnavailable, BrowserIncorrectPassword, BrowserForbidden, \
+                              BrowserSSLError, BrowserQuestion, BrowserHTTPSDowngrade, \
+                              ModuleInstallError, ModuleLoadError, NoAccountsException, \
+                              ActionNeeded, CaptchaQuestion
 from weboob.tools.value import Value, ValueBool, ValueFloat, ValueInt, ValueBackendPassword
 from weboob.tools.misc import to_unicode
-from weboob.tools.ordereddict import OrderedDict
+from weboob.tools.compat import unicode, long
 
 from .base import Application, MoreResultsAvailable
 
@@ -47,7 +51,7 @@ class BackendNotGiven(Exception):
     def __init__(self, id, backends):
         self.id = id
         self.backends = sorted(backends)
-        Exception.__init__(self, 'Please specify a backend to use for this argument (%s@backend_name). '
+        super(BackendNotGiven, self).__init__('Please specify a backend to use for this argument (%s@backend_name). '
                 'Availables: %s.' % (id, ', '.join(name for name, backend in backends)))
 
 
@@ -55,6 +59,25 @@ class BackendNotFound(Exception):
     pass
 
 
+class ConsoleProgress(IProgress):
+    def __init__(self, app):
+        self.app = app
+
+    def progress(self, percent, message):
+        try:
+            quiet = self.app.options.quiet
+        except AttributeError:
+            quiet = False
+        if not quiet:
+            self.app.stdout.write('=== [%3.0f%%] %s\n' % (percent*100, message))
+
+    def error(self, message):
+        self.app.stderr.write('ERROR: %s\n' % message)
+
+    def prompt(self, message):
+        return self.app.ask(message, default=True)
+
+
 class ConsoleApplication(Application):
     """
     Base application class for CLI applications.
@@ -74,9 +97,11 @@ class ConsoleApplication(Application):
         NC     = '[0m'    # no color
 
     def __init__(self, option_parser=None):
-        Application.__init__(self, option_parser)
-        self.weboob.callbacks['login'] = self.login_cb
+        super(ConsoleApplication, self).__init__(option_parser)
+        self.weboob.requests.register('login', self.login_cb)
         self.enabled_backends = set()
+        self._parser.add_option('--auto-update', action='store_true',
+                                help='Automatically check for updates when a bug in a module is encountered')
 
     def login_cb(self, backend_name, value):
         return self.ask('[%s] %s' % (backend_name,
@@ -87,7 +112,7 @@ class ConsoleApplication(Application):
 
     def unload_backends(self, *args, **kwargs):
         unloaded = self.weboob.unload_backends(*args, **kwargs)
-        for backend in unloaded.itervalues():
+        for backend in unloaded.values():
             try:
                 self.enabled_backends.remove(backend)
             except KeyError:
@@ -105,12 +130,12 @@ class ConsoleApplication(Application):
         ret = super(ConsoleApplication, self).load_backends(*args, **kwargs)
 
         for err in errors:
-            print('Error(%s): %s' % (err.backend_name, err), file=self.stderr)
+            print(u'Error(%s): %s' % (err.backend_name, err), file=self.stderr)
             if self.ask('Do you want to reconfigure this backend?', default=True):
                 self.edit_backend(err.backend_name)
                 self.load_backends(names=[err.backend_name])
 
-        for name, backend in ret.iteritems():
+        for name, backend in ret.items():
             self.enabled_backends.add(backend)
 
         self.check_loaded_backends()
@@ -132,7 +157,7 @@ class ConsoleApplication(Application):
         while r != 'q':
             modules = []
             print('\nAvailable modules:')
-            for name, info in sorted(self.weboob.repositories.get_all_modules_info().iteritems()):
+            for name, info in sorted(self.weboob.repositories.get_all_modules_info().items()):
                 if not self.is_module_loadable(info):
                     continue
                 modules.append(name)
@@ -146,7 +171,8 @@ class ConsoleApplication(Application):
                         else:
                             loaded += 1
                 print('%s%d)%s [%s] %s%-15s%s   %s' % (self.BOLD, len(modules), self.NC, loaded,
-                                                       self.BOLD, name, self.NC, info.description))
+                                                       self.BOLD, name, self.NC,
+                                                       info.description))
             print('%sa) --all--%s               install all backends' % (self.BOLD, self.NC))
             print('%sq)%s --stop--\n' % (self.BOLD, self.NC))
             r = self.ask('Select a backend to create (q to stop)', regexp='^(\d+|q|a)$')
@@ -158,7 +184,7 @@ class ConsoleApplication(Application):
                     continue
                 name = modules[i]
                 try:
-                    inst = self.add_backend(name, default_config)
+                    inst = self.add_backend(name, name, default_config)
                     if inst:
                         self.load_backends(names=[inst])
                 except (KeyboardInterrupt, EOFError):
@@ -168,7 +194,7 @@ class ConsoleApplication(Application):
                     for name in modules:
                         if name in [b.NAME for b in self.weboob.iter_backends()]:
                             continue
-                        inst = self.add_backend(name, default_config)
+                        inst = self.add_backend(name, name, default_config)
                         if inst:
                             self.load_backends(names=[inst])
                 except (KeyboardInterrupt, EOFError):
@@ -237,7 +263,7 @@ class ConsoleApplication(Application):
     def register_backend(self, name, ask_add=True):
         try:
             backend = self.weboob.modules_loader.get_or_load_module(name)
-        except ModuleLoadError as e:
+        except ModuleLoadError:
             backend = None
 
         if not backend:
@@ -256,7 +282,7 @@ class ConsoleApplication(Application):
             website = 'with backend %s' % backend.name
         while True:
             asked_config = False
-            for key, prop in backend.klass.ACCOUNT_REGISTER_PROPERTIES.iteritems():
+            for key, prop in backend.klass.ACCOUNT_REGISTER_PROPERTIES.items():
                 if not asked_config:
                     asked_config = True
                     print('Configuration of new account %s' % website)
@@ -277,18 +303,18 @@ class ConsoleApplication(Application):
             else:
                 break
         backend_config = {}
-        for key, value in account.properties.iteritems():
+        for key, value in account.properties.items():
             if key in backend.config:
                 backend_config[key] = value.get()
 
         if ask_add and self.ask('Do you want to add the new register account?', default=True):
-            return self.add_backend(name, backend_config, ask_register=False)
+            return self.add_backend(name, name, backend_config, ask_register=False)
 
         return backend_config
 
     def install_module(self, name):
         try:
-            self.weboob.repositories.install(name)
+            self.weboob.repositories.install(name, ConsoleProgress(self))
         except ModuleInstallError as e:
             print('Unable to install module "%s": %s' % (name, e), file=self.stderr)
             return False
@@ -297,9 +323,9 @@ class ConsoleApplication(Application):
         return True
 
     def edit_backend(self, name, params=None):
-        return self.add_backend(name, params, True)
+        return self.add_backend(name, name, params, True)
 
-    def add_backend(self, name, params=None, edit=False, ask_register=True):
+    def add_backend(self, module_name, backend_name, params=None, edit=False, ask_register=True):
         if params is None:
             params = {}
 
@@ -307,27 +333,27 @@ class ConsoleApplication(Application):
         config = None
         try:
             if not edit:
-                minfo = self.weboob.repositories.get_module_info(name)
+                minfo = self.weboob.repositories.get_module_info(module_name)
                 if minfo is None:
-                    raise ModuleLoadError(name, 'Module does not exist')
+                    raise ModuleLoadError(module_name, 'Module does not exist')
                 if not minfo.is_installed():
                     print('Module "%s" is available but not installed.' % minfo.name)
                     self.install_module(minfo)
-                module = self.weboob.modules_loader.get_or_load_module(name)
+                module = self.weboob.modules_loader.get_or_load_module(module_name)
                 config = module.config
             else:
-                bname, items = self.weboob.backends_config.get_backend(name)
-                module = self.weboob.modules_loader.get_or_load_module(bname)
+                module_name, items = self.weboob.backends_config.get_backend(backend_name)
+                module = self.weboob.modules_loader.get_or_load_module(module_name)
                 items.update(params)
                 params = items
-                config = module.config.load(self.weboob, bname, name, params, nofail=True)
+                config = module.config.load(self.weboob, module_name, backend_name, params, nofail=True)
         except ModuleLoadError as e:
-            print('Unable to load module "%s": %s' % (name, e), file=self.stderr)
+            print('Unable to load module "%s": %s' % (module_name, e), file=self.stderr)
             return 1
 
         # ask for params non-specified on command-line arguments
         asked_config = False
-        for key, value in config.iteritems():
+        for key, value in config.items():
             if not asked_config:
                 asked_config = True
                 print('')
@@ -336,28 +362,32 @@ class ConsoleApplication(Application):
             if key not in params or edit:
                 params[key] = self.ask(value, default=params[key] if (key in params) else value.default)
             else:
-                print(u' [%s] %s: %s' % (key, value.description, '(masked)' if value.masked else to_unicode(params[key])))
+                print(u'[%s] %s: %s' % (key, value.description, '(masked)' if value.masked else to_unicode(params[key])))
         if asked_config:
             print('-------------------------%s' % ('-' * len(module.name)))
 
-        while not edit and self.weboob.backends_config.backend_exists(name):
-            print('Backend instance "%s" already exists in "%s"' % (name, self.weboob.backends_config.confpath), file=self.stderr)
-            if not self.ask('Add new backend for module "%s"?' % module.name, default=False):
+        i = 2
+        while not edit and self.weboob.backends_config.backend_exists(backend_name):
+            if not self.ask('Backend "%s" already exists. Add a new one for module %s?' % (backend_name, module.name), default=False):
                 return 1
 
-            name = self.ask('Please give new instance name', default='%s2' % name, regexp=r'^[\w\-_]+$')
+            backend_name = backend_name.rstrip('0123456789')
+            while self.weboob.backends_config.backend_exists('%s%s' % (backend_name, i)):
+                i += 1
+            backend_name = self.ask('Please give new instance name', default='%s%s' % (backend_name, i), regexp=r'^[\w\-_]+$')
 
         try:
-            config = config.load(self.weboob, module.name, name, params, nofail=True)
-            for key, value in params.iteritems():
-                if key.startswith('_'):
+            config = config.load(self.weboob, module.name, backend_name, params, nofail=True)
+            for key, value in params.items():
+                if key not in config:
                     continue
                 config[key].set(value)
+
             config.save(edit=edit)
-            print('Backend "%s" successfully %s.' % (name, 'edited' if edit else 'added'))
-            return name
+            print('Backend "%s" successfully %s.' % (backend_name, 'edited' if edit else 'added'))
+            return backend_name
         except BackendAlreadyExists:
-            print('Backend "%s" already exists.' % name, file=self.stderr)
+            print('Backend "%s" already exists.' % backend_name, file=self.stderr)
             return 1
 
     def ask(self, question, default=None, masked=None, regexp=None, choices=None, tiny=None):
@@ -398,11 +428,13 @@ class ConsoleApplication(Application):
             v = klass(label=question, default=default, masked=masked, regexp=regexp, choices=choices, tiny=tiny)
 
         question = v.label
+        if v.description and v.description != v.label:
+            question = u'%s: %s' % (question, v.description)
         if v.id:
             question = u'[%s] %s' % (v.id, question)
 
         if isinstance(v, ValueBackendPassword):
-            print(question.encode(self.encoding) + ':')
+            print(question + ':')
             question = v.label
             choices = OrderedDict()
             choices['c'] = 'Run an external tool during backend load'
@@ -429,7 +461,7 @@ class ConsoleApplication(Application):
                 while True:
                     cmd = self.ask('')
                     try:
-                        subprocess.check_output(cmd, shell=True)
+                        check_output(cmd, shell=True)
                     except subprocess.CalledProcessError as e:
                         print('%s' % e)
                     else:
@@ -441,21 +473,21 @@ class ConsoleApplication(Application):
         elif v.choices:
             if v.tiny is None:
                 v.tiny = True
-                for key in v.choices.iterkeys():
+                for key in v.choices:
                     if len(key) > 5 or ' ' in key:
                         v.tiny = False
                         break
 
             if v.tiny:
                 question = u'%s (%s)' % (question, '/'.join((s.upper() if s == v.default else s)
-                                                            for s in v.choices.iterkeys()))
-                for s in v.choices.iterkeys():
+                                                            for s in v.choices))
+                for s in v.choices:
                     if s == v.default:
                         aliases[s.upper()] = s
-                for key, value in v.choices.iteritems():
+                for key, value in v.choices.items():
                     print('     %s%s%s: %s' % (self.BOLD, key, self.NC, value))
             else:
-                for n, (key, value) in enumerate(v.choices.iteritems()):
+                for n, (key, value) in enumerate(v.choices.items()):
                     print('     %s%2d)%s %s' % (self.BOLD, n + 1, self.NC, value))
                     aliases[str(n + 1)] = key
                 question = u'%s (choose in list)' % question
@@ -469,14 +501,17 @@ class ConsoleApplication(Application):
 
         while True:
             if v.masked:
-                if sys.platform == 'win32':
-                    line = getpass.getpass(str(question))
-                else:
-                    line = getpass.getpass(question.encode(self.encoding))
+                if sys.version_info.major < 3 and isinstance(question, unicode):
+                    question = question.encode(self.encoding)
+
+                line = getpass.getpass(question)
+                if sys.platform != 'win32':
+                    if isinstance(line, bytes): # only for python2
+                        line = line.decode(self.encoding)
             else:
-                self.stdout.write(question.encode(self.encoding))
+                self.stdout.write(question)
                 self.stdout.flush()
-                line = self.stdin.readline()
+                line = self._readline()
                 if len(line) == 0:
                     raise EOFError()
                 else:
@@ -484,8 +519,6 @@ class ConsoleApplication(Application):
 
             if not line and v.default is not None:
                 line = v.default
-            if isinstance(line, str):
-                line = line.decode('utf-8')
 
             if line in aliases:
                 line = aliases[line]
@@ -500,6 +533,21 @@ class ConsoleApplication(Application):
         v.noprompt = True
         return v.get()
 
+    def print(self, txt):
+        print(txt)
+
+    def _readall(self):
+        if sys.version_info.major == 2:
+            return self.stdin.read().decode(self.encoding)
+        else:
+            return self.stdin.read()
+
+    def _readline(self):
+        if sys.version_info.major == 2:
+            return self.stdin.readline().decode(self.encoding)
+        else:
+            return self.stdin.readline()
+
     def acquire_input(self, content=None, editor_params=None):
         editor = os.getenv('EDITOR', 'vi')
         if self.stdin.isatty() and editor:
@@ -508,7 +556,7 @@ class ConsoleApplication(Application):
                 filename = f.name
                 if content is not None:
                     if isinstance(content, unicode):
-                        content = content.encode(self.encoding)
+                        content = content
                     f.write(content)
                     f.flush()
                 try:
@@ -522,16 +570,23 @@ class ConsoleApplication(Application):
             if self.stdin.isatty():
                 print('Reading content from stdin... Type ctrl-D '
                           'from an empty line to stop.')
-            text = self.stdin.read()
-        return text.decode(self.encoding)
+            text = self._readall()
+        return to_unicode(text)
 
     def bcall_error_handler(self, backend, error, backtrace):
         """
         Handler for an exception inside the CallErrors exception.
 
-        This method can be overrided to support more exceptions types.
+        This method can be overridden to support more exceptions types.
         """
-        if isinstance(error, BrowserIncorrectPassword):
+        if isinstance(error, BrowserQuestion):
+            for field in error.fields:
+                v = self.ask(field)
+                if v:
+                    backend.config[field.id].set(v)
+        elif isinstance(error, CaptchaQuestion):
+            print(u'Warning(%s): Captcha has been found on login page' % backend.name, file=self.stderr)
+        elif isinstance(error, BrowserIncorrectPassword):
             msg = unicode(error)
             if not msg:
                 msg = 'invalid login/password.'
@@ -540,37 +595,46 @@ class ConsoleApplication(Application):
                 self.unload_backends(names=[backend.name])
                 self.edit_backend(backend.name)
                 self.load_backends(names=[backend.name])
+        elif isinstance(error, BrowserSSLError):
+            print(u'FATAL(%s): ' % backend.name + self.BOLD + '/!\ SERVER CERTIFICATE IS INVALID /!\\' + self.NC, file=self.stderr)
+        elif isinstance(error, BrowserHTTPSDowngrade):
+            print(u'FATAL(%s): ' % backend.name + 'Downgrade from HTTPS to HTTP')
+        elif isinstance(error, BrowserForbidden):
+            msg = unicode(error)
+            print(u'Error(%s): %s' % (backend.name, msg or 'Forbidden'), file=self.stderr)
         elif isinstance(error, BrowserUnavailable):
             msg = unicode(error)
-            if not msg:
-                msg = 'website is unavailable.'
-            print(u'Error(%s): %s' % (backend.name, msg), file=self.stderr)
-        elif isinstance(error, BrowserForbidden):
-            print(u'Error(%s): %s' % (backend.name, msg or 'Forbidden'), file=self.stderr)
+            print(u'Error(%s): %s' % (backend.name, msg or 'Website is unavailable.'), file=self.stderr)
+        elif isinstance(error, ActionNeeded):
+            msg = unicode(error)
+            print(u'Error(%s): Action needed on website: %s' % (backend.name, msg), file=self.stderr)
         elif isinstance(error, NotImplementedError):
             print(u'Error(%s): this feature is not supported yet by this backend.' % backend.name, file=self.stderr)
             print(u'      %s   To help the maintainer of this backend implement this feature,' % (' ' * len(backend.name)), file=self.stderr)
-            print(u'      %s   please contact: %s <%s@issues.weboob.org>' % (' ' * len(backend.name), backend.MAINTAINER, backend.NAME), file=self.stderr)
+            print(u'      %s   please contact us on the project mailing list' % (' ' * len(backend.name)), file=self.stderr)
         elif isinstance(error, UserError):
             print(u'Error(%s): %s' % (backend.name, to_unicode(error)), file=self.stderr)
         elif isinstance(error, MoreResultsAvailable):
             print(u'Hint: There are more results for backend %s' % (backend.name), file=self.stderr)
-        elif isinstance(error, BrowserSSLError):
-            print(u'FATAL(%s): ' % backend.name + self.BOLD + '/!\ SERVER CERTIFICATE IS INVALID /!\\' + self.NC, file=self.stderr)
+        elif isinstance(error, NoAccountsException):
+            print(u'Error(%s): %s' % (backend.name, to_unicode(error) or 'No account on this backend'), file=self.stderr)
         else:
             print(u'Bug(%s): %s' % (backend.name, to_unicode(error)), file=self.stderr)
 
             minfo = self.weboob.repositories.get_module_info(backend.NAME)
             if minfo and not minfo.is_local():
-                self.weboob.repositories.update_repositories()
+                if self.options.auto_update:
+                    self.weboob.repositories.update_repositories(ConsoleProgress(self))
 
-                # minfo of the new available module
-                minfo = self.weboob.repositories.get_module_info(backend.NAME)
-                if minfo and minfo.version > self.weboob.repositories.versions.get(minfo.name) and \
-                   self.ask('A new version of %s is available. Do you want to install it?' % minfo.name, default=True) and \
-                   self.install_module(minfo):
-                    print('New version of module %s has been installed. Retry to call the command.' % minfo.name)
-                    return
+                    # minfo of the new available module
+                    minfo = self.weboob.repositories.get_module_info(backend.NAME)
+                    if minfo and minfo.version > self.weboob.repositories.versions.get(minfo.name) and \
+                       self.ask('A new version of %s is available. Do you want to install it?' % minfo.name, default=True) and \
+                       self.install_module(minfo):
+                        print('New version of module %s has been installed. Retry to call the command.' % minfo.name)
+                        return
+                else:
+                    print('(If --auto-update is passed on the command-line, new versions of the module will be checked automatically)')
 
             if logging.root.level <= logging.DEBUG:
                 print(backtrace, file=self.stderr)
@@ -583,15 +647,21 @@ class ConsoleApplication(Application):
         """
         ask_debug_mode = False
         more_results = set()
+        err = 0
+
         for backend, error, backtrace in errors.errors:
             if isinstance(error, MoreResultsAvailable):
                 more_results.add(backend.name)
             elif isinstance(error, ignore):
                 continue
-            elif self.bcall_error_handler(backend, error, backtrace):
-                ask_debug_mode = True
+            else:
+                err = 1
+                if self.bcall_error_handler(backend, error, backtrace):
+                    ask_debug_mode = True
 
         if ask_debug_mode:
             print(debugmsg, file=self.stderr)
         elif len(more_results) > 0:
             print('Hint: There are more results available for %s (use option -n or count command)' % (', '.join(more_results)), file=self.stderr)
+
+        return err
